const express = require('express');
const mysql = require('mysql');
const blood_request = express();
// Middleware for parsing JSON
blood_request.use(express.json());
//DB configure-connection
const connection = require('../config');


//Getby All method 
blood_request.get('/blood_request', (req, res) => {
  const query = 'SELECT * FROM blood_request';

  connection.query(query, (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ', err);
      res.sendStatus(500);
      return;
    }

    res.json(results);
  });
});

// GET by Id method
blood_request.get('/request/:id', (req, res) => {
  const requestId = req.params.id;
  const query = 'SELECT * FROM blood_request WHERE id = ?';

  connection.query(query, [requestId], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ', err);
      res.sendStatus(500);
      return;
    }

    if (results.length === 0) {
      res.sendStatus(404);
      return;
    }

    res.json(results[0]);
  });
});


// POST method
blood_request.post('/add-request', (req, res) => {
  const {
    created_by,
    patient_name,
    patient_age,
    patient_medical_note,
    needed_blood_group,
    alternative_blood_group_1,
    alternative_blood_group_2,
    alternative_blood_group_3,
    alternative_blood_group_4,
    alternative_blood_group_5,
    alternative_blood_group_6,
    no_of_blood_unit,
    have_you_related_with_patient,
    blood_need_date,
    document_path,
    hospital_name,
    hospital_address,
    hospital_city,
    hospital_state,
    hospital_country,
    request_status
  } = req.body;

  const query = `INSERT INTO blood_request (
    created_by,
    patient_name,
    patient_age,
    patient_medical_note,
    needed_blood_group,
    alternative_blood_group_1,
    alternative_blood_group_2,
    alternative_blood_group_3,
    alternative_blood_group_4,
    alternative_blood_group_5,
    alternative_blood_group_6,
    no_of_blood_unit,
    have_you_related_with_patient,
    blood_need_date,
    document_path,
    hospital_name,
    hospital_address,
    hospital_city,
    hospital_state,
    hospital_country,
    request_status,
    created_at,
    updated_at
  ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, NOW(), NOW())`;

  connection.query(
    query,
    [
      created_by,
      patient_name,
      patient_age,
      patient_medical_note,
      needed_blood_group,
      alternative_blood_group_1,
      alternative_blood_group_2,
      alternative_blood_group_3,
      alternative_blood_group_4,
      alternative_blood_group_5,
      alternative_blood_group_6,
      no_of_blood_unit,
      have_you_related_with_patient,
      blood_need_date,
      document_path,
      hospital_name,
      hospital_address,
      hospital_city,
      hospital_state,
      hospital_country,
      request_status
    ],
    (err, results) => {
      if (err) {
        console.error('Error executing MySQL query: ', err);
        res.sendStatus(500);
        return;
      }

      res.sendStatus(200);
    }
  );
});

// PUT method
blood_request.put('/edit-request/:id', (req, res) => {
  const requestId = req.params.id;
  const {
    created_by,
    patient_name,
    patient_age,
    patient_medical_note,
    needed_blood_group,
    alternative_blood_group_1,
    alternative_blood_group_2,
    alternative_blood_group_3,
    alternative_blood_group_4,
    alternative_blood_group_5,
    alternative_blood_group_6,
    no_of_blood_unit,
    have_you_related_with_patient,
    blood_need_date,
    document_path,
    hospital_name,
    hospital_address,
    hospital_city,
    hospital_state,
    hospital_country,
    request_status
  } = req.body;

  const query = `UPDATE blood_request SET
    created_by = ?,
    patient_name = ?,
    patient_age = ?,
    patient_medical_note = ?,
    needed_blood_group = ?,
    alternative_blood_group_1 = ?,
    alternative_blood_group_2 = ?,
    alternative_blood_group_3 = ?,
    alternative_blood_group_4 = ?,
    alternative_blood_group_5 = ?,
    alternative_blood_group_6 = ?,
    no_of_blood_unit = ?,
    have_you_related_with_patient = ?,
    blood_need_date = ?,
    document_path = ?,
    hospital_name = ?,
    hospital_address = ?,
    hospital_city = ?,
    hospital_state = ?,
    hospital_country = ?,
    request_status = ?,
    updated_at = NOW()
    WHERE id = ?`;

  connection.query(
    query,
    [
      created_by,
      patient_name,
      patient_age,
      patient_medical_note,
      needed_blood_group,
      alternative_blood_group_1,
      alternative_blood_group_2,
      alternative_blood_group_3,
      alternative_blood_group_4,
      alternative_blood_group_5,
      alternative_blood_group_6,
      no_of_blood_unit,
      have_you_related_with_patient,
      blood_need_date,
      document_path,
      hospital_name,
      hospital_address,
      hospital_city,
      hospital_state,
      hospital_country,
      request_status,
      requestId
    ],
    (err, results) => {
      if (err) {
        console.error('Error executing MySQL query: ', err);
        res.sendStatus(500);
        return;
      }

      if (results.affectedRows === 0) {
        res.sendStatus(404);
        return;
      }

      res.sendStatus(200);
    }
  );
});

// DELETE method
blood_request.delete('/delete-request/:id', (req, res) => {
  const requestId = req.params.id;
  const query = 'DELETE FROM blood_request WHERE id = ?';

  connection.query(query, [requestId], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ', err);
      res.sendStatus(500);
      return;
    }

    if (results.affectedRows === 0) {
      res.sendStatus(404);
      return;
    }

    res.sendStatus(200);
  });
});
//GET relation all
blood_request.get('/bloodrequestrelate', (req, res) => {
  // Query to fetch all users' information and matching profiles
  const query = 'SELECT u.*, br.* FROM users u LEFT JOIN blood_request br ON u.id = br.created_by';

  connection.query(query, (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ', err);
      res.sendStatus(500);
      return;
    }

    if (results.length === 0) {
      res.sendStatus(404);
      return;
    }

    const bloodrequests = [];

    results.forEach(result => {
      const bloodrequest = {
        id: result.id,
        name: result.name,
        mobile_number: result.mobile_number,
        date_of_birth: result.date_of_birth,
        gender: result.gender,
        email: result.email,
        matching_profiles: []
      };

      if (result.created_by) {
        const info = {
          id: result.id,
          created_by: result.created_by,
          patient_name: result.patient_name,
          patient_age: result.patient_age,
          patient_medical_note: result.patient_medical_note,
          needed_blood_group: result.needed_blood_group,
          alternative_blood_group_1: result.alternative_blood_group_1,
          alternative_blood_group_2: result.alternative_blood_group_2,
          alternative_blood_group_3: result.alternative_blood_group_3,
          alternative_blood_group_4: result.alternative_blood_group_4,
          alternative_blood_group_5: result.alternative_blood_group_5,
          alternative_blood_group_6: result.alternative_blood_group_6,
          no_of_blood_unit: result.no_of_blood_unit,
          have_you_related_with_patient: result.have_you_related_with_patient,
          blood_need_date: result.blood_need_date,
          document_path: result.document_path,
          hospital_name: result.hospital_name,
          hospital_address: result.hospital_address,
          hospital_city: result.hospital_city,
          hospital_state: result.hospital_state,
          hospital_country: result.hospital_country,
          request_status: result.request_status,
          created_at: result.created_at,
          updated_at: result.updated_at
        };

        bloodrequest.matching_profiles.push(info);
      }

      bloodrequests.push(bloodrequest);
    });

    res.json(bloodrequests);
  });
});

// GET method to list all matching user_id in users_donar_profile
blood_request.get('/bloodrequestrelate/:user_id', (req, res) => {
  const createdby = req.params.user_id;

  // Query to fetch the user's information and matching profiles
  const query = 'SELECT u.*, br.* FROM users u LEFT JOIN blood_request br ON u.id = br.created_by WHERE u.id = ?';

  connection.query(query, [createdby], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ', err);
      res.sendStatus(500);
      return;
    }

    if (results.length === 0) {
      res.sendStatus(404);
      return;
    }

    const bloodrequest = {
      id: results[0].id,
      name: results[0].name,
      mobile_number: results[0].mobile_number,
      date_of_birth: results[0].date_of_birth,
      gender: results[0].gender,
      email: results[0].email,
      matching_profiles: []
    };

    results.forEach(result => {
      if (result.created_by) {
        const info = {
          id: result.id,
          created_by: result.created_by,
          patient_name: result.patient_name,
          patient_age: result.patient_age,
          patient_medical_note: result.patient_medical_note,
          needed_blood_group: result.needed_blood_group,
          alternative_blood_group_1: result.alternative_blood_group_1,
          alternative_blood_group_2: result.alternative_blood_group_2,
          alternative_blood_group_3: result.alternative_blood_group_3,
          alternative_blood_group_4: result.alternative_blood_group_4,
          alternative_blood_group_5: result.alternative_blood_group_5,
          alternative_blood_group_6: result.alternative_blood_group_6,
          no_of_blood_unit: result.no_of_blood_unit,
          have_you_related_with_patient: result.have_you_related_with_patient,
          blood_need_date: result.blood_need_date,
          document_path: result.document_path,
          hospital_name: result.hospital_name,
          hospital_address: result.hospital_address,
          hospital_city: result.hospital_city,
          hospital_state: result.hospital_state,
          hospital_country: result.hospital_country,
          request_status: result.request_status,
          created_at: result.created_at,
          updated_at: result.updated_at
        };

        bloodrequest.matching_profiles.push(info);
      }
    });

    res.json(bloodrequest);
  });
});

module.exports = blood_request;