const express = require('express');
const multer = require('multer');
const ftp = require('basic-ftp');
const claim_details = express();
const crypto = require('crypto');
const { Readable } = require('stream'); // Import the Readable class from the 'stream' module
// Middleware for parsing JSON
claim_details.use(express.json());
const connection = require('../config');

// Configure multer for handling file uploads
const storage = multer.memoryStorage();
const upload = multer({ storage }); //Define a directory to store uploaded files
// FTP Configuration
const ftpConfig = {
  host: 'senthuli.com', // FTP server hostname
  user: 'fileuser@files.senthuli.com', // FTP username
  password: 'fileuser@', // FTP password
  secure: false, // Use explicit FTPS (Set to true for FTPS)
  port: 21, // FTP port (usually 21 for FTP, but check if it's configured differently)
};

// Now you can use ftpConfig in your FTP client setup to connect to the server.

// FTP Upload function
// FTP Upload function
async function uploadFileToFTP(fileBuffer, originalFileName) {
  const client = new ftp.Client();

  try {
    await client.access(ftpConfig);

    // Generate a unique filename by appending a random string
    const uniqueFileName = generateUniqueFileName(originalFileName);

    // Create a Readable Stream from the file buffer
    const readableStream = new Readable();
    readableStream.push(fileBuffer);
    readableStream.push(null);

    // Upload the file to FTP with the unique filename
    await client.uploadFrom(readableStream, uniqueFileName);
  } catch (err) {
    console.error('FTP Upload Error:', err);
  } finally {
    client.close();
  }
}

// Generate a unique filename by appending a random string
// Generate a unique filename by adding a timestamp
function generateUniqueFileName(originalFileName) {
  const timestamp = Date.now();
  const fileExtension = originalFileName.split('.').pop();
  const originalNameWithoutExtension = originalFileName.replace(/\.[^/.]+$/, ''); // Remove the extension
  return `${originalNameWithoutExtension}.${fileExtension}`;
}



// Handle file upload request
// Handle file upload request
claim_details.post('/ftpupload', upload.single('file'), async (req, res) => {
  if (!req.file) {
    return res.status(400).send('No file uploaded.');
  }

  const { originalname, buffer } = req.file;

  // Extract the file extension
  const fileExtension = originalname.split('.').pop();
  const originalNameWithoutExtension = originalname.replace(/\.[^/.]+$/, ''); // Remove the extension

  // Generate a timestamp
  const timestamp = Date.now();

  // Create a new filename with the original filename and timestamp
  const newFileName = `${originalNameWithoutExtension}_${timestamp}.${fileExtension}`;

  try {
    // Upload the file to FTP with the new filename
    await uploadFileToFTP(buffer, newFileName);

    // Include the new filename in the response data
    const responseData = {
      message: 'File uploaded successfully to FTP.',
      file_name: newFileName // Use the new filename with timestamp
    };

    res.status(200).json(responseData);
  } catch (error) {
    console.error('Error uploading file to FTP:', error);
    res.status(500).json({ message: 'Internal server error' });
  }
});




//Get method
claim_details.get('/claim_details', (req, res) => {
  const query = 'SELECT * FROM claim_details';

  connection.query(query, (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to retrieve coupon' });
      return;
    }

    if (results.length === 0) {
      res.status(404).json({ error: 'No coupon found' });
    } else {
      res.json(results);
    }
  });
});
//get by claimed_by id
// Get method for retrieving a claim by claimed_by
claim_details.get('/get-claim_details/:claimed_by', (req, res) => {
  // Extract the claimed_by parameter from the URL
  const claimedBy = req.params.claimed_by;

  // SQL query to select claims with the specified claimed_by
  const query = 'SELECT * FROM claim_details WHERE claimed_by = ?';

  connection.query(query, [claimedBy], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to retrieve claim' });
      return;
    }

    if (results.length === 0) {
      res.status(404).json({ error: 'No claim found for claimed_by: ' + claimedBy });
    } else {
      res.json(results);
    }
  });
});

//get by id
claim_details.get('/claim_details/:id', (req, res) => {   //retreiving all the shop names where created_by
  const id = req.params.id;

  // Execute the SQL query to retrieve shop names for the provided created_by user ID
  const query = 'SELECT * FROM claim_details WHERE id = ?';
  connection.query(query, [id], (err, result) => {
    if (err) {
      console.error('Error retrieving Mysql query:', err);
      return res.status(500).json({ error: 'Internal server error' });
    }
    return res.json(result);
  });
});

// Post method
claim_details.post('/upload',(req, res) => {

    const {claim_doc,donor_name,claimed_by, donor_mobile, donor_blood_group,patient_name,blood_donated_date,raiser_name,raiser_mobile,raiser_email,is_verify,response_id,is_claimed } = req.body;
    
  
    // Insert file information into the MySQL database
    const query = 'INSERT INTO claim_details (claim_doc,claimed_by,donor_name, donor_mobile, donor_blood_group,patient_name,blood_donated_date,raiser_name,raiser_mobile,raiser_email,is_verify,response_id,is_claimed,created_at, updated_at) VALUES (?,?,?,?,?, ?, ?, ?,?,?, ?, ?, ?, NOW(), NOW())';
    connection.query(query, [claim_doc,claimed_by,donor_name, donor_mobile, donor_blood_group,patient_name,blood_donated_date,raiser_name,raiser_mobile,raiser_email,is_verify,response_id,is_claimed], (err, result) => {
      if (err) {
        console.error('Error inserting data into the database: ', err);
        return res.status(500).json({ error: 'Internal Server Error' });
      }
      
      // Get the ID of the newly inserted record
      const insertedId = result.insertId;
  
      // Create a response object with all the data, including the ID
      const responseData = {
        id: insertedId,
        claim_doc,
        claimed_by,
        donor_name,
         donor_mobile, 
         donor_blood_group,
         patient_name,
         blood_donated_date,
         raiser_name,
         raiser_mobile,
         raiser_email,
         is_verify,
         response_id,
         is_claimed,
        created_at: new Date(), // You can use a proper date format here
        updated_at: new Date(), // You can use a proper date format here
      };
  
      res.status(200).json(responseData); // 200 status code indicates resource creation
  
    });
  });
  
// PUT method for updating a record
claim_details.put('/update/:id', upload.single('claim_doc'), (req, res) => {
  const claimId = req.params.id; // Get the ID of the claim to be updated

  const {
    donor_name,
    claimed_by,
    donor_mobile,
    donor_blood_group,
    patient_name,
    blood_donated_date,
    raiser_name,
    raiser_mobile,
    raiser_email,
    is_verify,
  } = req.body;

  let imageFilePath;
  if (req.file) {
    // If a new image is uploaded, update the imageFilePath
    imageFilePath = req.file.buffer.toString('base64');
  }

  // Construct the SQL UPDATE query
  const updateQuery =
    'UPDATE claim_details SET ' +
    'claim_doc = COALESCE(?, claim_doc), ' +
    'claimed_by = ?, ' +
    'donor_name = ?, ' +
    'donor_mobile = ?, ' +
    'donor_blood_group = ?, ' +
    'patient_name = ?, ' +
    'blood_donated_date = ?, ' +
    'raiser_name = ?, ' +
    'raiser_mobile = ?, ' +
    'raiser_email = ?, ' +
    'is_verify = ?, ' +
    'updated_at = NOW() ' +
    'WHERE id = ?';

  const queryParams = [
    imageFilePath,
    claimed_by,
    donor_name,
    donor_mobile,
    donor_blood_group,
    patient_name,
    blood_donated_date,
    raiser_name,
    raiser_mobile,
    raiser_email,
    is_verify,
    claimId,
  ];

  connection.query(updateQuery, queryParams, (err, result) => {
    if (err) {
      console.error('Error updating data in the database: ', err);
      return res.status(500).json({ error: 'Internal Server Error' });
    }

    // Check if a record was updated
    if (result.affectedRows === 0) {
      return res.status(404).json({ error: 'Claim not found' });
    }

    // Create a response object with all the data
    const responseData = {
      id: claimId,
      claim_doc: imageFilePath || 'Image not updated', // Provide appropriate message if image not updated
      claimed_by,
      donor_name,
      donor_mobile,
      donor_blood_group,
      patient_name,
      blood_donated_date,
      raiser_name,
      raiser_mobile,
      raiser_email,
      is_verify,
      updated_at: new Date(), // You can use a proper date format here
    };

    res.status(200).json(responseData); // 200 status code indicates success
  });
});

// DELETE method for deleting a coupon by ID
claim_details.delete('/delete/:id', (req, res) => {
    const couponId = req.params.id; // Get the ID from the URL parameter

    // Delete the coupon from the MySQL database
    const deleteQuery = 'DELETE FROM claim_details WHERE id = ?';

    connection.query(deleteQuery, [couponId], (err, result) => {
        if (err) {
            console.error('Error deleting data from the database: ', err);
            return res.status(500).json({ error: 'Internal Server Error' });
        }

        if (result.affectedRows === 0) {
            // No rows were deleted (coupon with given ID not found)
            return res.status(404).json({ error: 'Coupon not found' });
        }

        res.status(200).json({ message: 'Coupon successfully deleted', deletedId: couponId });
    });
});


module.exports = claim_details;