const express = require('express');
const mysql = require('mysql');
const core_team = express();
// Middleware for parsing JSON
core_team.use(express.json());
//DB configure-connection
const connection = require('../config');


//GET from all users 
core_team.get('/get', (req, res) => {
    const query = 'SELECT * FROM core_team';
  
    connection.query(query, (err, results) => {
      if (err) {
        console.error('Error executing MySQL query: ' + err.stack);
        res.status(500).json({ error: 'Failed to retrieve users' });
        return;
      }
  
      if (results.length === 0) {
        res.status(404).json({ error: 'No users found' });
      } else {
        res.json(results);
      }
    });
  });

  //GET by Id method
  core_team.get('/get/:id', (req, res) => {
    const userId = req.params.id;
    const query = 'SELECT * FROM core_team WHERE id = ?';
  
    connection.query(query, [userId], (err, results) => {
      if (err) {
        console.error('Error executing MySQL query: ' + err.stack);
        res.status(500).json({ error: 'Failed to retrieve user' });
        return;
      }
  
      if (results.length === 0) {
        res.status(404).json({ error: 'User not found' });
      } else {
        res.json(results[0]);
      }
    });
  });

// POST method
core_team.post('/post', (req, res) => {
    const {
      name,
      company_name,
      occupation,
      location,
      profile_image
    } = req.body;
  
    const query = 'INSERT INTO core_team (name, company_name, occupation, location, profile_image, created_at, updated_at) VALUES (?, ?, ?, ?, ?,NOW(), NOW())';
  
    const parameters = [
      name,
      company_name,
      occupation,
      location,
      profile_image
    ];
  
    connection.query(query, parameters, (err, results) => {
      if (err) {
        console.error('Error executing MySQL query: ' + err.stack);
        res.status(500).json({ error: 'Failed to create user' });
        return;
      }
  
  
      res.json({message: 'User created successfully' });
    });
  });


//PUT method
core_team.put('/put/:id', (req, res) => {
    const Id = req.params.id;
    const updatedData = req.body;
  
    const query = 'UPDATE core_team SET ? WHERE id = ?';
    connection.query(query, [updatedData, Id], (error, results) => {
      if (error) {
        console.error('Error executing query:', error);
        res.status(500).json({ error: 'Error executing query' });
      } else {
        if (results.affectedRows > 0) {
          res.json({ message: 'Data updated successfully' });
        } else {
          res.status(404).json({ error: 'Book not found' });
        }
      }
    });
  });


  // DELETE method
  core_team.delete('/delete/:id', (req, res) => {
    const userId = req.params.id;
    const query = 'DELETE FROM core_team WHERE id = ?';
    connection.query(query, [userId], (err, results) => {
      if (err) {
        console.error('Error executing MySQL query: ', err);
        res.sendStatus(500);
        return;
      }
  
      if (results.affectedRows === 0) {
        res.sendStatus(404);
        return;
      }
  
      res.sendStatus(200);
    });
  });

module.exports = core_team;