const express = require('express');
const multer = require('multer');
const coupon_details = express();
// Middleware for parsing JSON
coupon_details.use(express.json());
const csv = require('csv-parser');
const connection = require('../config');
const storage = multer.memoryStorage();
const upload = multer({ storage: storage });


//create bulk
coupon_details.post('/bulk-uploadCoupon', upload.single('csvFile'), (req, res) => {
  if (!req.file) {
    return res.status(400).send('No file uploaded.');
  }

  const fileBuffer = req.file.buffer.toString();
  const records = fileBuffer.split('\n');

  for (const record of records) {
    // Split the CSV record into columns
    const [coupon_text] = record.split(','); // Assuming the first column is coupon_text

    // Check if coupon_text is empty, and if it is, skip this iteration
    if (!coupon_text.trim()) {
      continue;
    }

    // Extract other columns from your manual payload data
    const { master_id, is_assigned, assigned_to, assigned_on, coupon_created_at } = req.body;

    const sql = 'INSERT INTO coupon_details (coupon_text, master_id, is_assigned, assigned_to, assigned_on, coupon_created_at) VALUES (?, ?, ?, ?, ?, ?)';

    connection.query(sql, [coupon_text, master_id, is_assigned, assigned_to, assigned_on, coupon_created_at], (err) => {
      if (err) {
        console.error(err);
      }
    });
  }

  res.send('CSV data uploaded and inserted into the database.');
});
//Get method
coupon_details.get('/coupon_details', (req, res) => {
  const query = 'SELECT * FROM coupon_details';

  connection.query(query, (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to retrieve coupon' });
      return;
    }

    if (results.length === 0) {
      res.status(404).json({ error: 'No coupon found' });
    } else {
      res.json(results);
    }
  });
});

//get by id
coupon_details.get('/coupon_details/:id', (req, res) => {   //retreiving all the shop names where created_by
  const id = req.params.id;
  // Execute the SQL query to retrieve shop names for the provided created_by user ID
  const query = 'SELECT * FROM coupon_details WHERE master_id = ?';
  connection.query(query, [id], (err, result) => {
    if (err) {
      console.error('Error retrieving Mysql query:', err);
      return res.status(500).json({ error: 'Internal server error' });
    }
    return res.json(result);
  });
});
//single-coupon
coupon_details.post('/coupon_details/post', (req, res) => {
    const { master_id, coupon_text, coupon_created_at, is_assigned,assigned_to, assigned_on } = req.body;
    // Validate request data (add your own validation as needed).
    connection.query(
      'INSERT INTO coupon_details (master_id, coupon_text, coupon_created_at, is_assigned,assigned_to, assigned_on) VALUES (?, ?, ?, ?, ?,?)',
      [master_id, coupon_text, coupon_created_at, is_assigned,assigned_to, assigned_on],
      (error, results) => {
        if (error) {
          console.error(error);
          res.status(500).json({ error: 'An error occurred while inserting data.' });
        } else {
          res.status(201).json({ message: 'Data inserted successfully.' });
        }
      }
    );
  });



// Update method
coupon_details.put('/coupon_details/put/:id', (req, res) => {
    const { master_id, coupon_text, coupon_created_at, is_assigned, assigned_to, assigned_on } = req.body;
    const { id } = req.params;

    // Validate request data (add your own validation as needed)

    // Update data in the MySQL database
    connection.query(
      'UPDATE coupon_details SET master_id=?, coupon_text=?, coupon_created_at=?, is_assigned=?, assigned_to=?, assigned_on=?,updated_at=NOW() WHERE id=?',
      [master_id, coupon_text, coupon_created_at, is_assigned, assigned_to, assigned_on, id],
      (error, results) => {
        if (error) {
          console.error(error);
          res.status(500).json({ error: 'An error occurred while updating data.' });
        } else if (results.affectedRows === 0) {
          res.status(404).json({ error: 'Coupon detail not found.' });
        } else {
          res.status(200).json({ message: 'Data updated successfully.' });
        }
      }
    );
});


// DELETE method for deleting a coupon by ID
coupon_details.delete('/delete/:id', (req, res) => {
    const couponId = req.params.id; // Get the ID from the URL parameter

    // Delete the coupon from the MySQL database
    const deleteQuery = 'DELETE FROM coupon_details WHERE id = ?';

    connection.query(deleteQuery, [couponId], (err, result) => {
        if (err) {
            console.error('Error deleting data from the database: ', err);
            return res.json(err).json({ error: 'Internal Server Error' });
        }

        if (result.affectedRows === 0) {
            // No rows were deleted (coupon with given ID not found)
            return res.status(404).json({ error: 'Coupon not found' });
        }

        res.status(200).json({ message: 'Coupon successfully deleted', deletedId: couponId });
    });
});


module.exports = coupon_details;