const express = require('express');
const mysql = require('mysql');
const coupon_transactions = express();
// Middleware for parsing JSON
coupon_transactions.use(express.json());

//DB configure-connection
const connection = require('../config');

// Middleware to parse JSON requests
coupon_transactions.use(express.json());

//Mobile api get bu claim api 

coupon_transactions.get('/coupon_transactions-by-claim/:claim_id', (req, res) => {
  const claimId = req.params.claim_id; // Get claim_id from the request parameters
  const query = 'SELECT * FROM coupon_transactions WHERE claim_id = ?';

  connection.query(query, [claimId], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to retrieve coupon_transactions' });
      return;
    }

    if (results.length === 0) {
      res.status(404).json({ error: 'No coupon_transactions found for claim_id ' + claimId });
      return;
    }

    // Extract the master_ids and coupon_ids from the results
    const masterIds = results.map(result => result.master_id);
    const couponIds = results.map(result => result.coupon_id);

    // Perform queries to fetch details from coupon_master and coupon_details
    const masterQuery = 'SELECT * FROM coupon_master WHERE id IN (?)';
    const couponDetailsQuery = 'SELECT * FROM coupon_details WHERE id IN (?)';

    connection.query(masterQuery, [masterIds], (err, masterResults) => {
      if (err) {
        console.error('Error executing MySQL query: ' + err.stack);
        res.status(500).json({ error: 'Failed to retrieve coupon_master details' });
        return;
      }

      connection.query(couponDetailsQuery, [couponIds], (err, couponDetailsResults) => {
        if (err) {
          console.error('Error executing MySQL query: ' + err.stack);
          res.status(500).json({ error: 'Failed to retrieve coupon_details' });
          return;
        }

        // Fetch user data based on assigned_to
        const assignedToIds = couponDetailsResults.map(detail => detail.assigned_to);
        const userQuery = 'SELECT * FROM users WHERE id IN (?)';

        connection.query(userQuery, [assignedToIds], (err, userResults) => {
          if (err) {
            console.error('Error executing MySQL query: ' + err.stack);
            res.status(500).json({ error: 'Failed to retrieve user data' });
            return;
          }

          // Now you have both coupon_transactions, coupon_master, coupon_details, and user details
          // You can combine and process the data as needed

          // Filter and format the data
          const currentDate = new Date();
          const combinedData = results.map(result => {
            const masterDetail = masterResults.find(master => master.id === result.master_id);
            const couponDetail = couponDetailsResults.find(detail => detail.id === result.coupon_id);
            const assignedUser = userResults.find(user => user.id === couponDetail.assigned_to);

            // Check if valid_start_date is today or greater than today
            const validStartDate = new Date(masterDetail.valid_start_date);
            if (validStartDate >= currentDate) {
              return {
                ...result,
                master_detail: {
                  ...masterDetail,
                  valid_start_date: validStartDate, // Update valid_start_date
                },
                coupon_detail: {
                  ...couponDetail,
                  assigned_user: assignedUser,
                },
              };
            } else {
              // Exclude items with invalid start dates
              return null;
            }
          }).filter(item => item !== null); // Filter out null entries

          res.json(combinedData);
        });
      });
    });
  });
});


//GET from all coupon_transactions 

coupon_transactions.get('/coupon_transactions-all', (req, res) => {
  const query = 'SELECT * FROM coupon_transactions';

  connection.query(query, (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to retrieve coupon_transactions' });
      return;
    }

    if (results.length === 0) {
      res.status(404).json({ error: 'No coupon_transactions found' });
      return;
    }

    // Extract the master_ids and coupon_ids from the results
    const masterIds = results.map(result => result.master_id);
    const couponIds = results.map(result => result.coupon_id);

    // Perform queries to fetch details from coupon_master and coupon_details
    const masterQuery = 'SELECT * FROM coupon_master WHERE id IN (?)';
    const couponDetailsQuery = 'SELECT * FROM coupon_details WHERE id IN (?)';

    connection.query(masterQuery, [masterIds], (err, masterResults) => {
      if (err) {
        console.error('Error executing MySQL query: ' + err.stack);
        res.status(500).json({ error: 'Failed to retrieve coupon_master details' });
        return;
      }

      connection.query(couponDetailsQuery, [couponIds], (err, couponDetailsResults) => {
        if (err) {
          console.error('Error executing MySQL query: ' + err.stack);
          res.status(500).json({ error: 'Failed to retrieve coupon_details' });
          return;
        }

        // Fetch user data based on assigned_to
        const assignedToIds = couponDetailsResults.map(detail => detail.assigned_to);
        const userQuery = 'SELECT * FROM users WHERE id IN (?)';

        connection.query(userQuery, [assignedToIds], (err, userResults) => {
          if (err) {
            console.error('Error executing MySQL query: ' + err.stack);
            res.status(500).json({ error: 'Failed to retrieve user data' });
            return;
          }

          // Now you have both coupon_transactions, coupon_master, coupon_details, and user details
          // You can combine and process the data as needed

          // For example, you can create a new data structure that includes all sets of data
          const combinedData = results.map(result => {
            const masterDetail = masterResults.find(master => master.id === result.master_id);
            const couponDetail = couponDetailsResults.find(detail => detail.id === result.coupon_id);
            const assignedUser = userResults.find(user => user.id === couponDetail.assigned_to);
            
            return {
              ...result,
              master_detail: masterDetail,
              coupon_detail: {
                ...couponDetail,
                assigned_user: assignedUser, // Add the assigned_user information
              },
            };
          });

          res.json(combinedData);
        });
      });
    });
  });
});


// GET by Id method
coupon_transactions.get('/coupon_transactions/:id', (req, res) => {
  const userId = req.params.id;
  const query = 'SELECT * FROM coupon_transactions WHERE id = ?';

  connection.query(query, [userId], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to retrieve user' });
      return;
    }

    if (results.length === 0) {
      res.status(404).json({ error: 'User not found' });
    } else {
      res.json(results[0]);
    }
  });
});


// POST method

// Assuming you have a connection to your database already established as 'connection'

function fetchAllMasterIds(callback) {
    const query = 'SELECT DISTINCT master_id, MIN(id) as coupon_id FROM coupon_details WHERE is_assigned <> 1 GROUP BY master_id';
  
    connection.query(query, (err, results) => {
      if (err) {
        console.error('Error executing MySQL query: ' + err.stack);
        callback(err, null);
        return;
      }
  
      callback(null, results);
    });
  }
  
  coupon_transactions.post('/add-coupon_transactions', (req, res) => {
    // Fetch all distinct master_ids along with their corresponding coupon_ids
    fetchAllMasterIds((err, masterCouponIds) => {
      if (err) {
        res.status(500).json({ error: 'Failed to fetch master_ids and coupon_ids' });
        return;
      }
  
      const {
        coupon_to_user_id,
        provided_date,
        claim_id
      } = req.body;
  
      // Create an array of arrays for bulk insert
      const currentDate = new Date(); // Current date for 'created_at' and 'updated_at'
      const bulkInsertValues = masterCouponIds.map(item => [
        item.master_id,
        item.coupon_id,
        coupon_to_user_id,
        provided_date,
        claim_id, // Add claim_detail_id here
        currentDate, // Set 'created_at' with current date
        currentDate // Set 'updated_at' with current date
      ]);
      
  
      const query = 'INSERT INTO coupon_transactions (master_id, coupon_id, coupon_to_user_id,  provided_date,claim_id, created_at, updated_at) VALUES ?';
  
      connection.query(query, [bulkInsertValues], (err, results) => {
        if (err) {
          console.error('Error executing MySQL query: ' + err.stack);
          res.status(500).json({ error: 'Failed to create user' });
          return;
        }
  
        // If the insert is successful, update is_assigned, assigned_on, and assigned_to for the current bulk insert
        const insertedCouponIds = bulkInsertValues.map(item => item[1]);
  
        const updateQuery = 'UPDATE coupon_details SET is_assigned = 1, assigned_on = ?, assigned_to = ? WHERE id IN (?)';
  
        connection.query(updateQuery, [currentDate, coupon_to_user_id, insertedCouponIds], (err, updateResults) => {
          if (err) {
            console.error('Error updating is_assigned: ' + err.stack);
            res.status(500).json({ error: 'Failed to update is_assigned' });
            return;
          }
  
          res.json({ message: 'Coupons added in transaction, is_assigned updated, assigned_on updated, and assigned_to updated' });
        });
      });
    });
  });
  

// PUT method
coupon_transactions.put('/edit-coupon_transactions/:id', (req, res) => {
  const userId = req.params.id;
  const { name, mobile_number, date_of_birth, gender, email, is_receipent, receipant_registered_date, is_donar, donar_registered_date } = req.body;
  const query = 'UPDATE coupon_transactions SET name = ?, mobile_number = ?, date_of_birth = ?, gender = ?, email = ?, is_receipent = ?, receipant_registered_date = ?, is_donar = ?, donar_registered_date = ?, updated_at = NOW() WHERE id = ?';

  connection.query(query, [name, mobile_number, date_of_birth, gender, email, is_receipent, receipant_registered_date, is_donar, donar_registered_date, userId], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to update user' });
      return;
    }

    if (results.affectedRows === 0) {
      res.status(404).json({ error: 'User not found' });
    } else {
      res.json({ message: 'User updated successfully' });
    }
  });
});

// DELETE method
coupon_transactions.delete('/delete-coupon_transactions/:id', (req, res) => {
  const userId = req.params.id;
  const query = 'DELETE FROM coupon_transactions WHERE id = ?';

  connection.query(query, [userId], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to delete user' });
      return;
    }

    if (results.affectedRows === 0) {
      res.status(404).json({ error: 'User not found' });
    } else {
      res.json({ message: 'User deleted successfully' });
    }
  });
});

module.exports = coupon_transactions;