const express = require('express');
const mysql = require('mysql');
const users = express();
// Middleware for parsing JSON
users.use(express.json());

//DB configure-connection
const connection = require('../config');

// Middleware to parse JSON requests
users.use(express.json());
//GET from all users 

users.get('/users', (req, res) => {
  const query = 'SELECT * FROM users';

  connection.query(query, (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to retrieve users' });
      return;
    }

    if (results.length === 0) {
      res.status(404).json({ error: 'No users found' });
    } else {
      res.json(results);
    }
  });
});

// GET by Id method
users.get('/users/:id', (req, res) => {
  const userId = req.params.id;
  const query = 'SELECT * FROM users WHERE id = ?';

  connection.query(query, [userId], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to retrieve user' });
      return;
    }

    if (results.length === 0) {
      res.status(404).json({ error: 'User not found' });
    } else {
      res.json(results[0]);
    }
  });
});


// POST method

users.post('/add-user', (req, res) => {
  const {
    name,
    mobile_number,
    date_of_birth,
    gender,
    email,
    is_receipent,
    receipant_registered_date,
    is_donar,
    donar_registered_date
  } = req.body;

  const query = 'INSERT INTO users (name, mobile_number, date_of_birth, gender, email, is_receipent, receipant_registered_date, is_donar, donar_registered_date, created_at, updated_at) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, NOW(), NOW())';

  const parameters = [
    name,
    mobile_number,
    date_of_birth,
    gender,
    email,
    is_receipent,
    receipant_registered_date,
    is_donar,
    donar_registered_date || null, // Use the donar_registered_date value if provided, or use null if it's not provided
  ];

  connection.query(query, parameters, (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to create user' });
      return;
    }

    const userId = results.insertId; // Get the ID of the newly created record

    res.json({ id: userId, message: 'User created successfully' });
  });
});


//Relation POST method


users.post('/addnewdoner', (req, res) => {
  const {
    name,
    mobile_number,
    date_of_birth,
    gender,
    email,
    is_receipent,
    receipant_registered_date,
    is_donar,
    donar_registered_date,
    blood_group,
    height_cm,
    weight_kg,
    country,
    state,
    city,
    prefered_city_for_donation,
    have_medical_issue,
    have_hiv_positive,
    having_history_covid_positive,
    have_tattoo_past_12_months,
    have_donated_in_last_6_months
  } = req.body;

  const insertUserQuery = `INSERT INTO users (name, mobile_number, date_of_birth, gender, email, is_receipent, receipant_registered_date, is_donar, donar_registered_date, created_at, updated_at) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, NOW(), NOW())`;

  connection.query(
    insertUserQuery,
    [
      name,
      mobile_number,
      date_of_birth,
      gender,
      email,
      is_receipent,
      receipant_registered_date,
      is_donar,
      donar_registered_date
    ],
    (userErr, userResults) => {
      if (userErr) {
        console.error('Error executing MySQL query for creating user: ' + userErr.stack);
        res.status(500).json({ error: 'Failed to create user' });
        return;
      }

      const userId = userResults.insertId; // Get the ID of the newly created user

      const insertDonorProfileQuery = `INSERT INTO users_donar_profile (
        user_id,
        blood_group,
        height_cm,
        weight_kg,
        country,
        state,
        city,
        prefered_city_for_donation,
        have_medical_issue,
        have_hiv_positive,
        having_history_covid_positive,
        have_tattoo_past_12_months,
        have_donated_in_last_6_months,
        created_at,
        updated_at
      ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, NOW(), NOW())`;

      connection.query(
        insertDonorProfileQuery,
        [
          userId,
          blood_group,
          height_cm,
          weight_kg,
          country,
          state,
          city,
          prefered_city_for_donation,
          have_medical_issue,
          have_hiv_positive,
          having_history_covid_positive,
          have_tattoo_past_12_months,
          have_donated_in_last_6_months
        ],
        (donorErr) => {
          if (donorErr) {
            console.error('Error executing MySQL query for creating donor profile: ', donorErr);
            res.status(500).json({ error: 'Failed to create donor profile' });
            return;
          }

          res.json({ id: userId, message: 'User and donor profile created successfully' });
        }
      );
    }
  );
});

//Relation PUT METHOD

//Relation PUT METHOD
users.put('/updatedoners/:userId', (req, res) => {
  const userId = req.params.userId; // Get the userId from the request parameters

  const {
    name,
    mobile_number,
    date_of_birth,
    gender,
    email,
    is_receipent,
    receipant_registered_date,
    is_donar,
    donar_registered_date,
    blood_group,
    height_cm,
    weight_kg,
    country,
    state,
    city,
    prefered_city_for_donation,
    have_medical_issue,
    have_hiv_positive,
    having_history_covid_positive,
    have_tattoo_past_12_months,
    have_donated_in_last_6_months
  } = req.body;

  const updateUserQuery = `UPDATE users SET 
    name = ?,
    mobile_number = ?,
    date_of_birth = ?,
    gender = ?,
    email = ?,
    is_receipent = ?,
    receipant_registered_date = ?,
    is_donar = ?,
    donar_registered_date = ?,
    updated_at = NOW()
    WHERE id = ?`;

  connection.query(
    updateUserQuery,
    [
      name,
      mobile_number,
      date_of_birth,
      gender,
      email,
      is_receipent,
      receipant_registered_date,
      is_donar,
      donar_registered_date,
      userId
    ],
    (userErr, userResults) => {
      if (userErr) {
        console.error('Error executing MySQL query for updating user: ' + userErr.stack);
        res.status(500).json({ error: 'Failed to update user' });
        return;
      }

      const updateDonorProfileQuery = `UPDATE users_donar_profile SET
        user_id = COALESCE(user_id, ?),
        blood_group = ?,
        height_cm = ?,
        weight_kg = ?,
        country = ?,
        state = ?,
        city = ?,
        prefered_city_for_donation = ?,
        have_medical_issue = ?,
        have_hiv_positive = ?,
        having_history_covid_positive = ?,
        have_tattoo_past_12_months = ?,
        have_donated_in_last_6_months = ?,
        updated_at = NOW()
        WHERE user_id = ''`;

      connection.query(
        updateDonorProfileQuery,
        [
          userId,
          blood_group,
          height_cm,
          weight_kg,
          country,
          state,
          city,
          prefered_city_for_donation,
          have_medical_issue,
          have_hiv_positive,
          having_history_covid_positive,
          have_tattoo_past_12_months,
          have_donated_in_last_6_months
        ],
        (donorErr) => {
          if (donorErr) {
            console.error('Error executing MySQL query for updating donor profile: ', donorErr);
            res.status(500).json({ error: 'Failed to update donor profile' });
            return;
          }

          res.json({ id: userId, message: 'User and donor profile updated successfully' });
        }
      );
    }
  );
});



users.put('/updateDoner/:userId', (req, res) => {
  const userId = req.params.userId; // Get the user ID from the request parameters

  const {
    name,
    mobile_number,
    date_of_birth,
    gender,
    email,
    is_receipent,
    receipant_registered_date,
    is_donar,
    donar_registered_date,
    blood_group,
    height_cm,
    weight_kg,
    country,
    state,
    city,
    prefered_city_for_donation,
    have_medical_issue,
    have_hiv_positive,
    having_history_covid_positive,
    have_tattoo_past_12_months,
    have_donated_in_last_6_months
  } = req.body;

  const updateUserQuery = `UPDATE users SET
    name = ?,
    mobile_number = ?,
    date_of_birth = ?,
    gender = ?,
    email = ?,
    is_receipent = ?,
    receipant_registered_date = ?,
    is_donar = ?,
    donar_registered_date = ?,
    updated_at = NOW()
    WHERE id = ?`;

  connection.query(
    updateUserQuery,
    [
      name,
      mobile_number,
      date_of_birth,
      gender,
      email,
      is_receipent,
      receipant_registered_date,
      is_donar,
      donar_registered_date,
      userId
    ],
    (userErr, userResults) => {
      if (userErr) {
        console.error('Error executing MySQL query for updating user: ' + userErr.stack);
        res.status(500).json({ error: 'Failed to update user' });
        return;
      }

      const updateDonorProfileQuery = `UPDATE users_donar_profile SET
        blood_group = ?,
        height_cm = ?,
        weight_kg = ?,
        country = ?,
        state = ?,
        city = ?,
        prefered_city_for_donation = ?,
        have_medical_issue = ?,
        have_hiv_positive = ?,
        having_history_covid_positive = ?,
        have_tattoo_past_12_months = ?,
        have_donated_in_last_6_months = ?,
        updated_at = NOW()
        WHERE user_id = ?`;

      connection.query(
        updateDonorProfileQuery,
        [
          blood_group,
          height_cm,
          weight_kg,
          country,
          state,
          city,
          prefered_city_for_donation,
          have_medical_issue,
          have_hiv_positive,
          having_history_covid_positive,
          have_tattoo_past_12_months,
          have_donated_in_last_6_months,
          userId
        ],
        (donorErr) => {
          if (donorErr) {
            console.error('Error executing MySQL query for updating donor profile: ', donorErr);
            res.status(500).json({ error: 'Failed to update donor profile' });
            return;
          }

          res.json({ id: userId, message: 'User and donor profile updated successfully' });
        }
      );
    }
  );
});

users.put('/updateDonors/:userId', (req, res) => {
  const userId = req.params.userId; // Get the user ID from the request parameters

  const {
    name,
    mobile_number,
    date_of_birth,
    gender,
    email,
    is_receipent,
    receipant_registered_date,
    is_donar,
    donar_registered_date,
  } = req.body;

  const updateUserQuery = `UPDATE users SET
    name = ?,
    mobile_number = ?,
    date_of_birth = ?,
    gender = ?,
    email = ?,
    is_receipent = ?,
    receipant_registered_date = ?,
    is_donar = ?,
    donar_registered_date = ?,
    updated_at = NOW()
    WHERE id = ?`;

  connection.query(
    updateUserQuery,
    [
      name,
      mobile_number,
      date_of_birth,
      gender,
      email,
      is_receipent,
      receipant_registered_date,
      is_donar,
      donar_registered_date,
      userId
    ],
    (userErr, userResults) => {
      if (userErr) {
        console.error('Error executing MySQL query for updating user: ' + userErr.stack);
        res.status(500).json({ error: 'Failed to update user' });
        return;
      }

      const {
        blood_group,
        height_cm,
        weight_kg,
        country,
        state,
        city,
        prefered_city_for_donation,
        have_medical_issue,
        have_hiv_positive,
        having_history_covid_positive,
        have_tattoo_past_12_months,
        have_donated_in_last_6_months
      } = req.body;

      const updateDonorProfileQuery = `INSERT INTO users_donar_profile (
        user_id,
        blood_group,
        height_cm,
        weight_kg,
        country,
        state,
        city,
        prefered_city_for_donation,
        have_medical_issue,
        have_hiv_positive,
        having_history_covid_positive,
        have_tattoo_past_12_months,
        have_donated_in_last_6_months,
        created_at,
        updated_at
      ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, NOW(), NOW())`;

      connection.query(
        updateDonorProfileQuery,
        [
          userId,
          blood_group,
          height_cm,
          weight_kg,
          country,
          state,
          city,
          prefered_city_for_donation,
          have_medical_issue,
          have_hiv_positive,
          having_history_covid_positive,
          have_tattoo_past_12_months,
          have_donated_in_last_6_months
        ],
        (donorErr) => {
          if (donorErr) {
            console.error('Error executing MySQL query for updating donor profile: ', donorErr);
            res.status(500).json({ error: 'Failed to update donor profile' });
            return;
          }

          res.json({ id: userId, message: 'User and donor profile updated successfully' });
        }
      );
    }
  );
});




// PUT method
users.put('/edit-users/:id', (req, res) => {
  const userId = req.params.id;
  const { name, mobile_number, date_of_birth, gender, email, is_receipent, receipant_registered_date, is_donar, donar_registered_date } = req.body;
  const query = 'UPDATE users SET name = ?, mobile_number = ?, date_of_birth = ?, gender = ?, email = ?, is_receipent = ?, receipant_registered_date = ?, is_donar = ?, donar_registered_date = ?, updated_at = NOW() WHERE id = ?';

  connection.query(query, [name, mobile_number, date_of_birth, gender, email, is_receipent, receipant_registered_date, is_donar, donar_registered_date, userId], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to update user' });
      return;
    }

    if (results.affectedRows === 0) {
      res.status(404).json({ error: 'User not found' });
    } else {
      res.json({ message: 'User updated successfully' });
    }
  });
});

// DELETE method
users.delete('/delete-users/:id', (req, res) => {
  const userId = req.params.id;
  const query = 'DELETE FROM users WHERE id = ?';

  connection.query(query, [userId], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ' + err.stack);
      res.status(500).json({ error: 'Failed to delete user' });
      return;
    }

    if (results.affectedRows === 0) {
      res.status(404).json({ error: 'User not found' });
    } else {
      res.json({ message: 'User deleted successfully' });
    }
  });
});

module.exports = users;