const express = require('express');
const mysql = require('mysql');
const users_donar_profile = express();
// Middleware for parsing JSON
users_donar_profile.use(express.json());
//DB configure-connection
const connection = require('../config');
// const app = express();


// GET all user method

users_donar_profile.get('/user', (req, res) => {
  const query = 'SELECT * FROM users_donar_profile';

  connection.query(query, (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ', err);
      res.sendStatus(500);
      return;
    }

    if (results.length === 0) {
      res.sendStatus(404);
      return;
    }

    res.json(results);
  });
});

// GET by Id method
users_donar_profile.get('/user/:id', (req, res) => {
  const userId = req.params.id;
  const query = 'SELECT * FROM users_donar_profile WHERE id = ?';

  connection.query(query, [userId], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ', err);
      res.sendStatus(500);
      return;
    }

    if (results.length === 0) {
      res.sendStatus(404);
      return;
    }

    res.json(results[0]);
  });
});

// POST method
users_donar_profile.post('/add-user', (req, res) => {
  const {
    user_id,
    blood_group,
    height_cm,
    weight_kg,
    country,
    state,
    city,
    prefered_city_for_donation,
    have_medical_issue,
    have_hiv_positive,
    having_history_covid_positive,
    have_tattoo_past_12_months,
    have_donated_in_last_6_months
  } = req.body;

  const query = `INSERT INTO users_donar_profile (
    user_id,
    blood_group,
    height_cm,
    weight_kg,
    country,
    state,
    city,
    prefered_city_for_donation,
    have_medical_issue,
    have_hiv_positive,
    having_history_covid_positive,
    have_tattoo_past_12_months,
    have_donated_in_last_6_months,
    created_at,
    updated_at
  ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, NOW(), NOW())`;

  connection.query(
    query,
    [
      user_id,
      blood_group,
      height_cm,
      weight_kg,
      country,
      state,
      city,
      prefered_city_for_donation,
      have_medical_issue,
      have_hiv_positive,
      having_history_covid_positive,
      have_tattoo_past_12_months,
      have_donated_in_last_6_months
    ],
    (err, results) => {
      if (err) {
        console.error('Error executing MySQL query: ', err);
        res.sendStatus(500);
        return;
      }

      res.sendStatus(200);
    }
  );
});


// PUT method
users_donar_profile.put('/edit-user/:id', (req, res) => {
  const {
    user_id,
    blood_group,
    height_cm,
    weight_kg,
    country,
    state,
    city,
    prefered_city_for_donation,
    have_medical_issue,
    have_hiv_positive,
    having_history_covid_positive,
    have_tattoo_past_12_months,
    have_donated_in_last_6_months
  } = req.body;

  const { id } = req.params;

  const query = `UPDATE users_donar_profile SET
    user_id = ?,
    blood_group = ?,
    height_cm = ?,
    weight_kg = ?,
    country = ?,
    state = ?,
    city = ?,
    prefered_city_for_donation = ?,
    have_medical_issue = ?,
    have_hiv_positive = ?,
    having_history_covid_positive = ?,
    have_tattoo_past_12_months = ?,
    have_donated_in_last_6_months = ?,
    updated_at = NOW()
  WHERE id = ?`;

  connection.query(
    query,
    [
      user_id,
      blood_group,
      height_cm,
      weight_kg,
      country,
      state,
      city,
      prefered_city_for_donation,
      have_medical_issue,
      have_hiv_positive,
      having_history_covid_positive,
      have_tattoo_past_12_months,
      have_donated_in_last_6_months,
      id
    ],
    (err, results) => {
      if (err) {
        console.error('Error executing MySQL query: ', err);
        res.sendStatus(500);
        return;
      }

      res.sendStatus(200);
    }
  );
});


// DELETE method
users_donar_profile.delete('/delete-user/:id', (req, res) => {
  const userId = req.params.id;
  const query = 'DELETE FROM users_donar_profile WHERE id = ?';

  connection.query(query, [userId], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ', err);
      res.sendStatus(500);
      return;
    }

    if (results.affectedRows === 0) {
      res.sendStatus(404);
      return;
    }

    res.sendStatus(200);
  });
});

// GET method to list all matching user_id in users_donar_profile
users_donar_profile.get('/userrelate/:user_id', (req, res) => {
  const userId = req.params.user_id;

  // Query to fetch the user's information and matching profiles
  const query = 'SELECT u.*, dp.* FROM users u LEFT JOIN users_donar_profile dp ON u.id = dp.user_id WHERE u.id = ?';


  connection.query(query, [userId], (err, results) => {
    if (err) {
      console.error('Error executing MySQL query: ', err);
      res.sendStatus(500);
      return;
    }

    if (results.length === 0) {
      res.sendStatus(404);
      return;
    }

    const user = {
      id: results[0].id,
      name: results[0].name,
      mobile_number: results[0].mobile_number,
      date_of_birth: results[0].date_of_birth,
      gender: results[0].gender,
      email: results[0].email,
      is_donar: results[0].is_donar,
      matching_profiles: []
    };

    results.forEach(result => {
      if (result.user_id) {
        const profile = {
          id: result.id,
          user_id: result.user_id,
          blood_group: result.blood_group,
          height_cm: result.height_cm,
          weight_kg: result.weight_kg,
          country: result.country,
          state: result.state,
          city: result.city,
          prefered_city_for_donation: result.prefered_city_for_donation,
          have_medical_issue: result.have_medical_issue,
          have_hiv_positive: result.have_hiv_positive,
          having_history_covid_positive: result.having_history_covid_positive,
          have_tattoo_past_12_months: result.have_tattoo_past_12_months,
          have_donated_in_last_6_months: result.have_donated_in_last_6_months,
          created_at: result.created_at,
          updated_at: result.updated_at
        };

        user.matching_profiles.push(profile);
      }
    });

    res.json(user);
  });
});

// combine post  
// Define POST route for creating a new record in users_donar_profile
// POST method for adding a user to users and users_donar_profile tables
users_donar_profile.post('/addnew', (req, res) => {
  const {
    user_id,
    blood_group,
    height_cm,
    weight_kg,
    country,
    state,
    city,
    prefered_city_for_donation,
    have_medical_issue,
    have_hiv_positive,
    having_history_covid_positive,
    have_tattoo_past_12_months,
    have_donated_in_last_6_months
  } = req.body;

  const query = `INSERT INTO users_donar_profile (
    user_id,
    blood_group,
    height_cm,
    weight_kg,
    country,
    state,
    city,
    prefered_city_for_donation,
    have_medical_issue,
    have_hiv_positive,
    having_history_covid_positive,
    have_tattoo_past_12_months,
    have_donated_in_last_6_months,
    created_at,
    updated_at,
    name,
    mobile_number,
    date_of_birth,
    gender,
    email,
    is_receipent,
    receipant_registered_date,
    is_donar,
    donar_registered_date
  ) SELECT
    ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?,
    NOW(), NOW(),
    u.name,
    u.mobile_number,
    u.date_of_birth,
    u.gender,
    u.email,
    u.is_receipent,
    u.receipant_registered_date,
    u.is_donar,
    u.donar_registered_date
  FROM
    users AS u
  WHERE
    u.user_id = ?`;

  connection.query(
    query,
    [
      user_id,
      blood_group,
      height_cm,
      weight_kg,
      country,
      state,
      city,
      prefered_city_for_donation,
      have_medical_issue,
      have_hiv_positive,
      having_history_covid_positive,
      have_tattoo_past_12_months,
      have_donated_in_last_6_months,
      user_id
    ],
    (err, results) => {
      if (err) {
        console.error('Error executing MySQL query: ', err);
        res.sendStatus(500);
        return;
      }

      res.sendStatus(200);
    }
  );
});
//// relational put method

users_donar_profile.put('/updateDonors-profile/:userId', (req, res) => {
  const userId = req.params.userId; // Get the user ID from the request parameters

  const {
    name,
    mobile_number,
    date_of_birth,
    gender,
    email,
    is_receipent,
    receipant_registered_date,
    is_donar,
    donar_registered_date,
  } = req.body;

  const updateUserQuery = `UPDATE users SET
    name = ?,
    mobile_number = ?,
    date_of_birth = ?,
    gender = ?,
    email = ?,
    is_receipent = ?,
    receipant_registered_date = ?,
    is_donar = ?,
    donar_registered_date = ?,
    updated_at = NOW()
    WHERE id = ?`;

  connection.query(
    updateUserQuery,
    [
      name,
      mobile_number,
      date_of_birth,
      gender,
      email,
      is_receipent,
      receipant_registered_date,
      is_donar,
      donar_registered_date,
      userId
    ],
    (userErr, userResults) => {
      if (userErr) {
        console.error('Error executing MySQL query for updating user: ' + userErr.stack);
        res.status(500).json({ error: 'Failed to update user' });
        return;
      }

      const {
        blood_group,
        height_cm,
        weight_kg,
        country,
        state,
        city,
        prefered_city_for_donation,
        have_medical_issue,
        have_hiv_positive,
        having_history_covid_positive,
        have_tattoo_past_12_months,
        have_donated_in_last_6_months
      } = req.body;

      const updateDonorProfileQuery = `UPDATE users_donar_profile SET
        blood_group = ?,
        height_cm = ?,
        weight_kg = ?,
        country = ?,
        state = ?,
        city = ?,
        prefered_city_for_donation = ?,
        have_medical_issue = ?,
        have_hiv_positive = ?,
        having_history_covid_positive = ?,
        have_tattoo_past_12_months = ?,
        have_donated_in_last_6_months = ?,
        updated_at = NOW()
        WHERE user_id = ?`;

      connection.query(
        updateDonorProfileQuery,
        [
          blood_group,
          height_cm,
          weight_kg,
          country,
          state,
          city,
          prefered_city_for_donation,
          have_medical_issue,
          have_hiv_positive,
          having_history_covid_positive,
          have_tattoo_past_12_months,
          have_donated_in_last_6_months,
          userId
        ],
        (donorErr) => {
          if (donorErr) {
            console.error('Error executing MySQL query for updating donor profile: ', donorErr);
            res.status(500).json({ error: 'Failed to update donor profile' });
            return;
          }

          res.json({ id: userId, message: 'User and donor profile updated successfully' });
        }
      );
    }
  );
});




module.exports = users_donar_profile;